/* $Id: init.c 46 2024-11-10 10:04:40Z nishi $ */

#include <MultiView/Module.h>
#include <MultiView/CommonLog.h>

#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

int MVMOD_PRIVATE(sizes)[][2] = {{640, 480}, {800, 600}, {1024, 768}, {1152, 870}, {1280, 1024}, {1400, 1050}, {1600, 1200}, {2048, 1536}, {2048, 1600}};

#define display MVMOD_PRIVATE(display)
#define window MVMOD_PRIVATE(window)
#define gc MVMOD_PRIVATE(gc)

Display* display;
Window window;
GC gc;

int module_init(void) {
	char buffer[512];
	int width, height, scrwidth, scrheight;
	int i;
	Atom atom;
	XSizeHints* size;
	width = -1;
	display = XOpenDisplay(NULL);
	if(display == NULL) {
		MULTIVIEW_PRIVATE(log)("X11", __MV_LOG_INFO, "OPENDISP", "Failed to open display");
		return MULTIVIEW_PRIVATE(MODULE_INIT_FAIL);
	}
	scrwidth = DisplayWidth(display, DefaultScreen(display));
	scrheight = DisplayHeight(display, DefaultScreen(display));
	sprintf(buffer, "Display size is %dx%d", scrwidth, scrheight);
	MULTIVIEW_PRIVATE(log)("X11", __MV_LOG_INFO, "DISPSIZE", buffer);
	for(i = sizeof(MVMOD_PRIVATE(sizes)) / sizeof(int[2]) - 1; i >= 0; i--) {
		if(MVMOD_PRIVATE(sizes)[i][0] < scrwidth && MVMOD_PRIVATE(sizes)[i][1] < scrheight) {
			width = MVMOD_PRIVATE(sizes)[i][0];
			height = MVMOD_PRIVATE(sizes)[i][1];
			break;
		}
	}
	if(width == -1) {
		MULTIVIEW_PRIVATE(log)("X11", __MV_LOG_INFO, "CALCSIZE", "Your display is too small to create a window");
		return MULTIVIEW_PRIVATE(MODULE_INIT_FAIL);
	} else {
		sprintf(buffer, "Window size is %dx%d", width, height);
		MULTIVIEW_PRIVATE(log)("X11", __MV_LOG_INFO, "CALCSIZE", buffer);
	}
	window = XCreateSimpleWindow(display, RootWindow(display, DefaultScreen(display)), 0, 0, width, height, 0, 0, 0);
	atom = XInternAtom(display, "WM_DELETE_WINDOW", 0);
	XSetWMProtocols(display, window, &atom, 1);
	size = XAllocSizeHints();
	size->flags = PMinSize | PMaxSize;
	size->min_width = size->max_width = width;
	size->min_height = size->max_height = height;
	XSetWMNormalHints(display, window, size);
	XFree(size);
	XStoreName(display, window, "MultiView Server (X11)");
	XMapWindow(display, window);
	XMoveWindow(display, window, (scrwidth - width) / 2, (scrheight - height) / 2);
	XFlush(display);
	gc = XCreateGC(display, DefaultRootWindow(display), 0, 0);
	MULTIVIEW_PRIVATE(log)("X11", __MV_LOG_INFO, "INIT", "Success");
	return MULTIVIEW_PRIVATE(MODULE_INIT_SUCCESS);
}
