/* $Id: config.c 35 2024-11-04 10:42:21Z nishi $ */

#include <MultiView/Server.h>
#include <MultiView/CommonLog.h>
#include <MultiView/Module.h>
#include <MultiView/Path.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <sys/stat.h>
#include <unistd.h>

int MVSERV_PRIVATE(parse_config)(const char* path) {
	FILE* f = fopen(path, "r");
	if(f == NULL) {
		MULTIVIEW_PRIVATE(log)("CONFIG", __MV_LOG_FATAL, "OPEN", "Could not open the config");
		return 1;
	} else {
		struct stat s;
		char* buffer;
		int i;
		int incr = 0;
		stat(path, &s);
		buffer = malloc(s.st_size + 1);
		fread(buffer, 1, s.st_size, f);
		buffer[s.st_size] = 0;

		for(i = 0;; i++) {
			if(buffer[i] == '\n' || buffer[i] == 0) {
				char oldc = buffer[i];
				char* line = buffer + incr;
				buffer[i] = 0;

				if(strlen(line) > 0 && line[0] != '#') {
					int j;
					char* cmd = line;
					char* arg = NULL;
					for(j = 0; line[j] != 0; j++) {
						if(line[j] == ' ' || line[j] == '\t') {
							line[j] = 0;
							j++;
							for(; line[j] != 0 && (line[j] == ' ' || line[j] == '\t'); j++);
							arg = line + j;
							if(strlen(line + j) == 0) arg = NULL;
							break;
						}
					}
					if(strcmp(cmd, "LoadDriver") == 0) {
						if(arg == NULL) {
							free(buffer);
							fclose(f);
							MULTIVIEW_PRIVATE(log)("CONFIG", __MV_LOG_FATAL, "LOADDRIVER", "LoadDriver requires an argument");
							return 1;
						}
						if(MVSERV_PRIVATE(load_module)(arg) == NULL) {
							free(buffer);
							fclose(f);
							MULTIVIEW_PRIVATE(log)("CONFIG", __MV_LOG_FATAL, "LOADDRIVER", "Failed to load the driver");
							return 1;
						}
					}
				}

				incr = i + 1;
				if(oldc == 0) break;
			}
		}
		free(buffer);

		fclose(f);
	}
	return 0;
}
