/* $Id: main.c 40 2024-11-10 05:27:50Z nishi $ */

#include <MultiView/Server.h>
#include <MultiView/Version.h>
#include <MultiView/CommonArg.h>
#include <MultiView/CommonLog.h>
#include <MultiView/Path.h>

#include <stdio.h>

MULTIVIEW_PRIVATE(args_t) args = {{'V', "version", 0}, {'C', "config", 1}};

int main(int argc, char** argv) {
	int ret;
	const char* confpath = __MultiView_ConfigPrefix "/MultiView.conf";
	char buffer[512];
	int fails;
	while((ret = MULTIVIEW_PRIVATE(getarg)(argv, args, 2)) != 0) {
		if(ret == -1) {
			fprintf(stderr, "Argument error\n");
			return 1;
		}
		switch(ret) {
		case 1:
			printf("MultiView server %s\n", MVVersion);
			return 0;
		case 2:
			confpath = MULTIVIEW_PRIVATE(getarg_option)()[0];
			break;
		}
	}
	MULTIVIEW_PRIVATE(log)("MAIN", __MV_LOG_INFO, "VERSION", "MultiView " MVVersion);
	sprintf(buffer, "Config path is %s", confpath);
	MULTIVIEW_PRIVATE(log)("MAIN", __MV_LOG_INFO, "CONFIG", buffer);
	if((ret = MVSERV_PRIVATE(parse_config)(confpath)) > 0) {
		MULTIVIEW_PRIVATE(log)("MAIN", __MV_LOG_FATAL, "CONFIG", "Failed to parse the config");
		return ret;
	}
	fails = MVSERV_PRIVATE(init_modules)();
	if(fails > 0) {
		sprintf(buffer, "%d module(s) reported failure", fails);
		MULTIVIEW_PRIVATE(log)("MAIN", __MV_LOG_FATAL, "INITMOD", buffer);
		return 1;
	}
}
