/* $Id: module.c 45 2024-11-10 08:32:28Z nishi $ */

#include <MultiView/Server.h>
#include <MultiView/Module.h>
#include <MultiView/CommonLog.h>
#include <MultiView/CommonString.h>
#include <MultiView/Path.h>

#include <stdlib.h>
#include <dlfcn.h>
#include <stdio.h>

struct MVSERV_PRIVATE(module) {
	void* lib;
};
typedef struct MVSERV_PRIVATE(module) MVSERV_PRIVATE(module_t);

MVSERV_PRIVATE(module_t) MVSERV_PRIVATE(modules)[64];
int MVSERV_PRIVATE(module_count) = 0;

void* MVSERV_PRIVATE(dlopen)(const char* path) { return dlopen(path, RTLD_NOW); }

void* MVSERV_PRIVATE(dlsym)(void* lib, const char* sym) { return dlsym(lib, sym); }

void* MVSERV_PRIVATE(load_module)(const char* path) {
	char* loadpath = (path[0] == '/' || path[0] == '.') ? MULTIVIEW_PRIVATE(strdup)(path) : MULTIVIEW_PRIVATE(strcat)(__MultiView_DriverPrefix "/", path);
	void* lib = MVSERV_PRIVATE(dlopen)(loadpath);
	char buffer[512];
	free(loadpath);
	if(lib == NULL) {
		sprintf(buffer, "Failed to load the module: %s", dlerror());
		MULTIVIEW_PRIVATE(log)("MODULE", __MV_LOG_WARN, "LOAD", buffer);
	} else {
		char buffer[512];
		MULTIVIEW_PRIVATE(module_get_info_t) get_info = (MULTIVIEW_PRIVATE(module_get_info_t))MVSERV_PRIVATE(dlsym)(lib, "module_get_info");
		buffer[0] = 0;
		sprintf(buffer, "Loaded module: %s version %s", get_info(MULTIVIEW_PRIVATE(MODULE_GET_NAME)), get_info(MULTIVIEW_PRIVATE(MODULE_GET_VERSION)));
		MULTIVIEW_PRIVATE(log)("MODULE", __MV_LOG_INFO, "LOAD", buffer);
		MVSERV_PRIVATE(modules)[MVSERV_PRIVATE(module_count)++].lib = lib;
	}
	return lib;
}

int MVSERV_PRIVATE(init_modules)(void) {
	int fails = 0;
	int i;
	for(i = 0; i < MVSERV_PRIVATE(module_count); i++) {
		MULTIVIEW_PRIVATE(module_init_t) init = (MULTIVIEW_PRIVATE(module_init_t))MVSERV_PRIVATE(dlsym)(MVSERV_PRIVATE(modules)[i].lib, "module_init");
		if(init != NULL) {
			if(init() == MULTIVIEW_PRIVATE(MODULE_INIT_FAIL)) fails++;
		}
	}
	return fails;
}
